/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;

public class Anzeige extends JComponent implements MouseListener {
	public Anzeige () {
		super();
		setSize (1000,200);

		anzeige = Tools.loadImage ("anzeige");

		initLampen();
		
		block_menu = new JPopupMenu();
		block_menu.add("Freie Bahn anfordern").addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				akt_block.freieBahnAngefordert();
			}
		});
		block_menu.add("Vorblocken").addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				akt_block.vorgeblockt();
			}
		});
		block_menu.add("Blocken").addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				akt_block.geblockt();
			}
		});
		block_menu.add("Rckmelden").addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				akt_block.rueckgemeldet();
			}
		});

		addMouseListener(this);
	}
	
	public void initBlock (Block l, Block r) {
		bf = new Bahnhof[2];
		bf[0] = new Bahnhof(l);
		bf[1] = new Bahnhof(r);
	}
	
	public final void initLampen () {
		l = new Lampe[30];
		l[0]  = new GleisLampe (this, 290, 110);	// Weiche 1
		l[1]  = new GleisLampe (this, 330, 110);	// Weiche 2
		l[2]  = new GleisLampe (this, 590, 110);	// Weiche 4
		l[3]  = new GleisLampe (this, 690, 110);	// Weiche 5
		l[4]  = new GleisLampe (this, 250, 110);	// Uebergang 1
		l[5]  = new GleisLampe (this, 330, 150);	// Gleis bei Weiche 1
		l[6]  = new GleisLampe (this, 650, 110);	// Gleis zwischen Weiche 4 und 5
		l[7]  = new GleisLampe (this, 650, 150);	// Gleis bei Weiche 5
		
		l[8]  = new Lampe (this, 150,  70, Color.RED);		// Signal A
		l[9]  = new Lampe (this, 170,  70, Color.GREEN);
		l[10] = new Lampe (this, 130,  70, Color.ORANGE);
		
		l[11] = new Lampe (this, 230, 150, Color.RED);		// Signal B
		l[12] = new Lampe (this, 210, 150, Color.GREEN);
		l[13] = new Lampe (this, 250, 150, Color.ORANGE);
		
		l[14] = new Lampe (this, 770,  70, Color.RED);		// Signal C
		l[15] = new Lampe (this, 790,  70, Color.GREEN);
		l[16] = new Lampe (this, 750,  70, Color.ORANGE);
		
		l[17] = new Lampe (this, 850, 150, Color.RED);		// Signal D
		l[18] = new Lampe (this, 830, 150, Color.GREEN);
		l[19] = new Lampe (this, 870, 150, Color.ORANGE);
		
		l[20] = new Lampe (this,  30,  70, Color.YELLOW);	// Signal A*
		l[21] = new Lampe (this, 970, 150, Color.YELLOW);	// Signal D*
		
		l[22] = new Blocklampe (this,  30, 110, Color.RED, Blocklampe.RECHTS);
		l[23] = new Blocklampe (this,  50, 110, Color.YELLOW, Blocklampe.RECHTS);
		l[24] = new Blocklampe (this,  90, 110, Color.RED, Blocklampe.LINKS);
		l[25] = new Blocklampe (this,  70, 110, Color.YELLOW, Blocklampe.LINKS);
		
		l[26] = new Blocklampe (this, 970, 110, Color.RED, Blocklampe.LINKS);
		l[27] = new Blocklampe (this, 950, 110, Color.YELLOW, Blocklampe.LINKS);
		l[28] = new Blocklampe (this, 910, 110, Color.RED, Blocklampe.RECHTS);
		l[29] = new Blocklampe (this, 930, 110, Color.YELLOW, Blocklampe.RECHTS);
		
		sig = new Signal[4];
		sig[0] = new Signal (l[8], l[9], l[10], l[20]);
		sig[1] = new Signal (l[11], l[12], l[13], null);
		sig[2] = new Signal (l[14], l[15], l[16], null);
		sig[3] = new Signal (l[17], l[18], l[19], l[21]);
		
	}
	
	@Override
	public void paint (Graphics g) {
		int i;
		g.drawImage (anzeige, 0, 0, this);
		for (i = 0; i < 30; i++) l[i].paint(g);
	}
	
	public void repaintLampen () {
//		repaint();
		int i;
		Graphics g = getGraphics();
		for (i = 0; i < 30; i++) l[i].paint(g);
	}
	
	public void setTestmodus (boolean ein) {
		testmodus_ein = ein;
	}
	
	public Lampe l[];
	public Signal sig[];
	
	BufferedImage anzeige;
	
	Bahnhof bf[];
	
	@Override
	public void mouseClicked (MouseEvent e) {
		int x;
		int y;
		int i;
		GleisLampe g;
		
		x = e.getX();
		y = e.getY();
		
		if (y > 100 && y < 120) {
			if (x < 80) {
				if (e.getButton() == MouseEvent.BUTTON1) {
					x *= 5;
					bf[0].sendeZug(x);
					return;
				} else {
					if (!testmodus_ein) return;
					akt_block = bf[0].block;
					block_menu.show(this, e.getX(), e.getY());
				}
			} else if (x > 920) {
				if (e.getButton() == MouseEvent.BUTTON1) {
					x = (1000-x) * 5;
					bf[1].sendeZug(x);
					return;
				} else {
					if (!testmodus_ein) return;
					akt_block = bf[1].block;
					block_menu.show(this, e.getX(), e.getY());
				}
			}
		}

		if (!testmodus_ein) return;

		x = x - (x % 20) + 10;
		y = y - (y % 20) + 10;
		
		for (i = 0; i < 8; i++) {
			g = (GleisLampe) l[i];
			if (g.p.x == x && g.p.y == y) {
				g.statusWechseln();
			}
		}
	}
	
	@Override public void mousePressed (MouseEvent e) { }
	@Override public void mouseReleased (MouseEvent e) { }
	@Override public void mouseEntered (MouseEvent e) { }
	@Override public void mouseExited (MouseEvent e) { }
	
	public boolean testmodus_ein;

	JPopupMenu block_menu;
	Block akt_block;
}
